@@ -9,7 +9,7 @@
 #include <QDir>
 #include <KConfigGroup>
 #include <KDesktopFile>
-#include <kmimetypetrader.h>
+#include <kapplicationtrader.h>
 #include <kservicefactory_p.h>
 #include <QTemporaryDir>
 #include <QTemporaryFile>
@@ -96,6 +96,17 @@ static void writeAppDesktopFile(const QString &path, const QStringList &mimeType
     group.writeXdgListEntry("MimeType", mimeTypes);
 }
 
+static void writeNonKDEAppDesktopFile(const QString &path, const QStringList &mimeTypes) // bug 427469
+{
+    KDesktopFile file(path);
+    KConfigGroup group = file.desktopGroup();
+    group.writeEntry("Name", "FakeApplication");
+    group.writeEntry("Type", "Application");
+    group.writeEntry("Exec", "ls");
+    group.writeEntry("NotShowIn", "KDE");
+    group.writeXdgListEntry("MimeType", mimeTypes);
+}
+
 /**
  * This unit test verifies the parsing of mimeapps.list files, both directly
  * and via kbuildsycoca (and making trader queries).
@@ -156,6 +167,12 @@ private Q_SLOTS:
         fakeOktetaApplication = m_localApps + "fakeoktetaapplication.desktop";
         writeAppDesktopFile(fakeOktetaApplication, QStringList() << QStringLiteral("application/octet-stream"));
 
+        const QString fakeGnomeRoller = m_localApps + "fake.org.gnome.FileRoller.desktop";
+        writeNonKDEAppDesktopFile(fakeGnomeRoller, QStringList() << QStringLiteral("application/x-7z-compressed"));
+
+        const QString fakeNautilus = m_localApps + "fake.org.gnome.Nautilus.desktop";
+        writeNonKDEAppDesktopFile(fakeNautilus, QStringList() << QStringLiteral("application/x-7z-compressed"));
+
         // Update ksycoca in ~/.qttest after creating the above
         runKBuildSycoca();
 
@@ -188,6 +205,8 @@ private Q_SLOTS:
                                  "x-scheme-handler/mailto=faketextapplication.desktop\n"
                                  // test association with octet-stream (#425154)
                                  "application/octet-stream=fakeoktetaapplication.desktop\n"
+                                 // test a non-kde app (#427469)
+                                 "application/x-7z-compressed=fake.org.gnome.FileRoller.desktop;\n"
                                  "[Added KParts/ReadOnlyPart Associations]\n"
                                  "text/plain=katepart.desktop;\n"
                                  "[Removed Associations]\n"
@@ -210,6 +229,8 @@ private Q_SLOTS:
         preferredApps[QStringLiteral("application/msword")] << QStringLiteral("fakecsrcmswordapplication.desktop");
         preferredApps[QStringLiteral("x-scheme-handler/mailto")] << QStringLiteral("faketextapplication.desktop");
         preferredApps[QStringLiteral("text/x-python")] << QStringLiteral("faketextapplication.desktop");
+        preferredApps[QStringLiteral("application/x-7z-compressed")] << QStringLiteral("fake.org.gnome.FileRoller.desktop");
+        removedApps[QStringLiteral("application/x-7z-compressed")] << QStringLiteral("fake.org.gnome.Nautilus.desktop");
         removedApps[QStringLiteral("image/jpeg")] << QStringLiteral("firefox.desktop");
         removedApps[QStringLiteral("text/html")] << QStringLiteral("gvim.desktop") << QStringLiteral("abiword.desktop");
 
@@ -333,20 +354,15 @@ private Q_SLOTS:
         writeToMimeApps(m_mimeAppsFileContents);
 
         // Test a trader query
-        KService::List offers = KMimeTypeTrader::self()->query(QStringLiteral("image/jpeg"));
+        KService::List offers = KApplicationTrader::queryByMimeType(QStringLiteral("image/jpeg"));
         QVERIFY(!offers.isEmpty());
-        //qDebug() << m_mimeAppsFileContents;
-        //qDebug() << "preferred apps for jpeg: " << preferredApps.value("image/jpeg");
-        //for (int i = 0; i < offers.count(); ++i) {
-        //    qDebug() << "offers for" << "image/jpeg" << ":" << i << offers[i]->storageId();
-        //}
         QCOMPARE(offers.first()->storageId(), QStringLiteral("fakejpegapplication.desktop"));
 
         // Now the generic variant of the above test:
         // for each mimetype, check that the preferred apps are as specified
         for (ExpectedResultsMap::const_iterator it = preferredApps.constBegin(), end = preferredApps.constEnd(); it != end; ++it) {
             const QString mime = it.key();
-            const KService::List offers = KMimeTypeTrader::self()->query(mime);
+            const KService::List offers = KApplicationTrader::queryByMimeType(mime);
             const QStringList offerIds = assembleServices(offers, it.value().count());
             if (offerIds != it.value()) {
                 qDebug() << "offers for" << mime << ":";
@@ -362,12 +378,22 @@ private Q_SLOTS:
             }
             QCOMPARE(offerIds, it.value());
         }
+        for (auto it = removedApps.constBegin(),
+                end = removedApps.constEnd(); it != end; ++it) {
+            const QString mime = it.key();
+            const KService::List offers = KApplicationTrader::queryByMimeType(mime);
+            const QStringList offerIds = assembleServices(offers);
+            for (const QString &service : it.value()) {
+                const QString error = QStringLiteral("Offers for %1 should not contain %2").arg(mime, service);
+                QVERIFY2(!offerIds.contains(service), qPrintable(error));
+            }
+        }
     }
 
     void testMultipleInheritance()
     {
         // application/x-shellscript inherits from both text/plain and application/x-executable
-        KService::List offers = KMimeTypeTrader::self()->query(QStringLiteral("application/x-shellscript"));
+        KService::List offers = KApplicationTrader::queryByMimeType(QStringLiteral("application/x-shellscript"));
         QVERIFY(offerListHasService(offers, fakeTextApplication, true));
     }
 
@@ -376,16 +402,16 @@ private Q_SLOTS:
         // I removed kate from text/plain, and it would still appear in text/x-java.
 
         // First, let's check our fake app is associated with text/plain
-        KService::List offers = KMimeTypeTrader::self()->query(QStringLiteral("text/plain"));
+        KService::List offers = KApplicationTrader::queryByMimeType(QStringLiteral("text/plain"));
         QVERIFY(offerListHasService(offers, fakeTextApplication, true));
 
         writeToMimeApps(QByteArray("[Removed Associations]\n"
                                    "text/plain=faketextapplication.desktop;\n"));
 
-        offers = KMimeTypeTrader::self()->query(QStringLiteral("text/plain"));
+        offers = KApplicationTrader::queryByMimeType(QStringLiteral("text/plain"));
         QVERIFY(!offerListHasService(offers, fakeTextApplication, false));
 
-        offers = KMimeTypeTrader::self()->query(QStringLiteral("text/x-java"));
+        offers = KApplicationTrader::queryByMimeType(QStringLiteral("text/x-java"));
         QVERIFY(!offerListHasService(offers, fakeTextApplication, false));
     }
 
@@ -402,16 +428,16 @@ private Q_SLOTS:
             QSKIP("Broken distro where application/vnd.oasis.opendocument.text doesn't inherit from application/zip");
         }
 
-        KService::List offers = KMimeTypeTrader::self()->query(opendocument);
+        KService::List offers = KApplicationTrader::queryByMimeType(opendocument);
         QVERIFY(offerListHasService(offers, fakeArkApplication, true));
 
         writeToMimeApps(QByteArray("[Removed Associations]\n"
                                    "application/vnd.oasis.opendocument.text=fakearkapplication.desktop;\n"));
 
-        offers = KMimeTypeTrader::self()->query(opendocument);
+        offers = KApplicationTrader::queryByMimeType(opendocument);
         QVERIFY(!offerListHasService(offers, fakeArkApplication, false));
 
-        offers = KMimeTypeTrader::self()->query(QStringLiteral("application/zip"));
+        offers = KApplicationTrader::queryByMimeType(QStringLiteral("application/zip"));
         QVERIFY(offerListHasService(offers, fakeArkApplication, true));
     }
 
@@ -430,16 +456,16 @@ private Q_SLOTS:
             QVERIFY(mime.inherits(QStringLiteral("application/zip")));
         }
 
-        KService::List offers = KMimeTypeTrader::self()->query(mime);
+        KService::List offers = KApplicationTrader::queryByMimeType(mime);
         QVERIFY(offerListHasService(offers, fakeArkApplication, true));
 
         writeToMimeApps(QByteArray("[Removed Associations]\n"
                                    "application/x-kns=fakearkapplication.desktop;\n"));
 
-        offers = KMimeTypeTrader::self()->query(mime);
+        offers = KApplicationTrader::queryByMimeType(mime);
         QVERIFY(!offerListHasService(offers, fakeArkApplication, false));
 
-        offers = KMimeTypeTrader::self()->query(QStringLiteral("application/zip"));
+        offers = KApplicationTrader::queryByMimeType(QStringLiteral("application/zip"));
         QVERIFY(offerListHasService(offers, fakeArkApplication, true));
     }
 
@@ -454,13 +480,13 @@ private Q_SLOTS:
         writeToMimeApps(QByteArray("[Removed Associations]\n"
                                    "application/x-desktop=faketextapplication.desktop;\n"));
 
-        offers = KMimeTypeTrader::self()->query(QStringLiteral("text/plain"));
+        offers = KApplicationTrader::queryByMimeType(QStringLiteral("text/plain"));
         QVERIFY(offerListHasService(offers, fakeTextApplication, true));
 
-        offers = KMimeTypeTrader::self()->query(QStringLiteral("application/x-desktop"));
+        offers = KApplicationTrader::queryByMimeType(QStringLiteral("application/x-desktop"));
         QVERIFY(!offerListHasService(offers, fakeTextApplication, false));
 
-        offers = KMimeTypeTrader::self()->query(QStringLiteral("application/x-theme"));
+        offers = KApplicationTrader::queryByMimeType(QStringLiteral("application/x-theme"));
         QVERIFY(!offerListHasService(offers, fakeTextApplication, false));
     }
 
