@@ -1,20 +1,8 @@
-/* This file is part of the KDE project
-   Copyright (C) 2015 David Faure <faure@kde.org>
+/*
+    This file is part of the KDE project
+    SPDX-FileCopyrightText: 2015 David Faure <faure@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include <ksycoca.h>
@@ -88,7 +76,7 @@ void KSycocaXdgDirsTest::testOtherAppDir()
     // Create another xdg data dir
     const QString dataDir = m_tempDir.path();
     qputenv("XDG_DATA_DIRS", QFile::encodeName(dataDir));
-    QCOMPARE(dataDir, QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).last());
+    QCOMPARE(dataDir, QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).constLast());
     QVERIFY(!KService::serviceByDesktopPath(QStringLiteral("test_app_other.desktop")));
 
     const QString appDir = dataDir + "/applications";
