@@ -1,36 +1,24 @@
-/* This file is part of the KDE libraries
-    Copyright (C) 2005 Brad Hards <bradh@frogmouth.net>
-    Copyright (C) 2006 Thiago Macieira <thiago@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2005 Brad Hards <bradh@frogmouth.net>
+    SPDX-FileCopyrightText: 2006 Thiago Macieira <thiago@kde.org>
 
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Library General Public
-    License as published by the Free Software Foundation; either
-    version 2 of the License, or (at your option) any later version.
-
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Library General Public License for more details.
-
-    You should have received a copy of the GNU Library General Public License
-    along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-    Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "ktoolinvocation.h"
 #ifdef QT_DBUS_LIB
 #include "klauncher_iface.h"
-#include <kdeinitinterface.h>
+#include <KDEInitInterface>
 #endif
-#include <klocalizedstring.h>
+#include <KLocalizedString>
 
 #include <QDebug>
 #include <QUrl>
 #include <QUrlQuery>
 #include <QCoreApplication>
 #include <QThread>
-#include <qstandardpaths.h>
+#include <QStandardPaths>
 
 #include <errno.h> // for EINVAL
 
@@ -75,10 +63,15 @@ bool KToolInvocation::isMainThreadActive(QString *error)
 }
 
 int KToolInvocation::startServiceInternal(const char *_function,
-        const QString &_name, const QStringList &URLs,
-        QString *error, QString *serviceName, int *pid,
-        const QByteArray &startup_id, bool noWait,
-        const QString &workdir)
+                                          const QString &_name,
+                                          const QStringList &URLs,
+                                          QString *error,
+                                          QString *serviceName,
+                                          int *pid,
+                                          const QByteArray &startup_id,
+                                          bool noWait,
+                                          const QString &workdir,
+                                          const QStringList &envs)
 {
 #ifdef QT_DBUS_LIB
     QString function = QLatin1String(_function);
@@ -92,10 +85,10 @@ int KToolInvocation::startServiceInternal(const char *_function,
         msg << workdir;
     }
     // make sure there is id, so that user timestamp exists
-    QStringList envs;
     QByteArray s = startup_id;
-    emit kapplication_hook(envs, s);
-    msg << envs;
+    QStringList envCopy(envs);
+    emit kapplication_hook(envCopy, s);
+    msg << envCopy;
     msg << QString::fromLatin1(s);
     if (!function.startsWith(QLatin1String("kdeinit_exec"))) {
         msg << noWait;
@@ -262,7 +255,9 @@ void KToolInvocation::invokeMailer(const QString &address, const QString &subjec
     invokeMailer(address, QString(), QString(), subject, QString(), QString(),
                  QStringList(), startup_id);
 }
+#endif
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
 void KToolInvocation::invokeMailer(const QUrl &mailtoURL, const QByteArray &startup_id, bool allowAttachments)
 {
     if (!isMainThreadActive()) {
