@@ -1,21 +1,9 @@
 /*
-   This file is part of the KDE libraries
-   Copyright (C) 2004-2008 Jarosław Staniek <staniek@kde.org>
-   Copyright (C) 2006 Ralf Habacker <ralf.habacker@freenet.de>
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2004-2008 Jarosław Staniek <staniek@kde.org>
+    SPDX-FileCopyrightText: 2006 Ralf Habacker <ralf.habacker@freenet.de>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License version 2 as published by the Free Software Foundation.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-only
 */
 
 #include "ktoolinvocation.h"
@@ -28,6 +16,7 @@
 #include <QProcess>
 #include <QCoreApplication>
 #include <QHash>
+#include <QStandardPaths>
 #include <QDBusConnection>
 
 #include "windows.h"
@@ -88,7 +77,47 @@ void KToolInvocation::invokeTerminal(const QString &command,
                                      const QString &workdir,
                                      const QByteArray &startup_id)
 {
-    //TODO
+    const QString windowsTerminal = QStringLiteral("wt.exe");
+    const QString pwsh = QStringLiteral("pwsh.exe");
+    const QString powershell = QStringLiteral("powershell.exe"); // Powershell is used as fallback
+    const bool hasWindowsTerminal = !QStandardPaths::findExecutable(windowsTerminal).isEmpty();
+    const bool hasPwsh = !QStandardPaths::findExecutable(pwsh).isEmpty();
+
+    QProcess process;
+    QStringList args;
+    process.setWorkingDirectory(workdir);
+
+    if (hasWindowsTerminal) {
+        process.setProgram(windowsTerminal);
+        if (!workdir.isEmpty()) {
+            args << QStringLiteral("--startingDirectory") << workdir;
+        }
+        if (!command.isEmpty()) {
+            // Command and NoExit flag will be added later
+            args << (hasPwsh ? pwsh : powershell);
+        }
+    } else {
+        process.setProgram(hasPwsh ? pwsh : powershell);
+    }
+    if (!command.isEmpty()) {
+        args << QStringLiteral("-NoExit") << QStringLiteral("-Command") << command;
+    }
+    process.setArguments(args);
+
+    if (!envs.isEmpty()) {
+        QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
+        for (const QString &envVar : envs) {
+            const int splitIndex = envVar.indexOf(QLatin1Char('='));
+            env.insert(envVar.left(splitIndex), envVar.mid(splitIndex + 1));
+        }
+        process.setProcessEnvironment(env);
+    }
+    process.setCreateProcessArgumentsModifier(
+                [](QProcess::CreateProcessArguments *args) {
+        args->flags |= CREATE_NEW_CONSOLE;
+        args->startupInfo->dwFlags &= ~STARTF_USESTDHANDLES;
+    });
+    process.startDetached();
 }
 
 void KToolInvocation::invokeTerminal(const QString &command, const QString &workdir, const QByteArray &startup_id)
