@@ -1,21 +1,10 @@
-/*  This file is part of the KDE libraries
- *  Copyright (C) 1999 - 2001 Waldo Bastian <bastian@kde.org>
- *  Copyright (C) 1999 - 2005 David Faure   <faure@kde.org>
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License version 2 as published by the Free Software Foundation;
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
- */
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 1999-2001 Waldo Bastian <bastian@kde.org>
+    SPDX-FileCopyrightText: 1999-2005 David Faure <faure@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-only
+*/
 
 #include "kservice.h"
 #include "kservice_p.h"
@@ -25,19 +14,18 @@
 
 #include <qplatformdefs.h>
 
-#include <QCharRef>
 #include <QDir>
 #include <QMap>
-#include <QCoreApplication>
-#include <qmimedatabase.h>
+#include <QMimeDatabase>
 
-#include <kauthorized.h>
-#include <kdesktopfile.h>
-#include <kconfiggroup.h>
-#include <klocalizedstring.h>
+#include <KAuthorized>
+#include <KDesktopFile>
+#include <KConfigGroup>
+#include <KLocalizedString>
+#include <KShell>
 
-#include <qstandardpaths.h>
-#include <qdebug.h>
+#include <QStandardPaths>
+#include <QDebug>
 
 #include "kservicefactory_p.h"
 #include "kservicetypefactory_p.h"
@@ -97,9 +85,9 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
           return;*/
         m_strType = QStringLiteral("Application");
     } else if (m_strType != QLatin1String("Application") && m_strType != QLatin1String("Service")) {
-        qCWarning(SERVICES) << "The desktop entry file " << entryPath
-                   << " has Type=" << m_strType
-                   << " instead of \"Application\" or \"Service\"";
+        qCWarning(SERVICES) << "The desktop entry file" << entryPath
+                   << "has Type=" << m_strType
+                   << "instead of \"Application\" or \"Service\"";
         m_bValid = false;
         return;
     }
@@ -159,7 +147,7 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     entryMap.remove(QStringLiteral("Terminal"));
     m_strTerminalOptions = desktopGroup.readEntry("TerminalOptions");   // should be a property IMHO
     entryMap.remove(QStringLiteral("TerminalOptions"));
-    m_strWorkingDirectory = config->readPath();
+    m_strWorkingDirectory = KShell::tildeExpand(config->readPath());
     entryMap.remove(QStringLiteral("Path"));
     m_strComment = config->readComment();
     entryMap.remove(QStringLiteral("Comment"));
@@ -214,7 +202,7 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
     while (st_it.hasNext()) {
         const QString st = st_it.next();
         if (st.isEmpty()) {
-            qCWarning(SERVICES) << "The desktop entry file" << entryPath << "has an empty mimetype!";
+            qCWarning(SERVICES) << "The desktop entry file" << entryPath << "has an empty MimeType!";
             continue;
         }
         int initialPreference = m_initialPreference;
@@ -246,6 +234,18 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
 
     m_strDesktopEntryName = _name.toLower();
 
+    // Exec lines from the KCMs always have the pattern "<program> m_strDesktopEntryName", see https://phabricator.kde.org/T13729
+    if (m_strExec.isEmpty() && serviceTypes().contains(QStringLiteral("KCModule"))) {
+        if (desktopGroup.readEntry("X-KDE-ParentApp") == QLatin1String("kinfocenter")) {
+            m_strExec = QStringLiteral("kinfocenter ") + m_strDesktopEntryName;
+        } else if (desktopGroup.readEntry("X-KDE-ParentApp") == QLatin1String("kcontrol")) {
+            if (!desktopGroup.readEntry("X-KDE-System-Settings-Parent-Category").isEmpty()) {
+                m_strExec = QStringLiteral("systemsettings5 ") + m_strDesktopEntryName;
+            } else {
+                m_strExec = QStringLiteral("kcmshell5 ") + m_strDesktopEntryName;
+            }
+        }
+    }
     m_bAllowAsDefault = desktopGroup.readEntry("AllowDefault", true);
     entryMap.remove(QStringLiteral("AllowDefault"));
 
@@ -268,7 +268,11 @@ void KServicePrivate::init(const KDesktopFile *config, KService *q)
         // do not store other translations like Name[fr]; kbuildsycoca will rerun if we change languages anyway
         if (!key.contains(QLatin1Char('['))) {
             //qCDebug(SERVICES) << "  Key =" << key << " Data =" << *it;
-            m_mapProps.insert(key, QVariant(*it));
+            if (key == QLatin1String("X-Flatpak-RenamedFrom")) {
+                m_mapProps.insert(key, desktopGroup.readXdgListEntry(key));
+            } else {
+                m_mapProps.insert(key, QVariant(*it));
+            }
         }
     }
 }
@@ -280,12 +284,14 @@ void KServicePrivate::parseActions(const KDesktopFile *config, KService *q)
         return;
     }
 
+    KService::Ptr serviceClone(new KService(*q));
+
     QStringList::ConstIterator it = keys.begin();
     const QStringList::ConstIterator end = keys.end();
     for (; it != end; ++it) {
         const QString group = *it;
         if (group == QLatin1String("_SEPARATOR_")) {
-            m_actions.append(KServiceAction(group, QString(), QString(), QString(), false));
+            m_actions.append(KServiceAction(group, QString(), QString(), QString(), false, serviceClone));
             continue;
         }
 
@@ -298,7 +304,7 @@ void KServicePrivate::parseActions(const KDesktopFile *config, KService *q)
                                                 cg.readEntry("Name"),
                                                 cg.readEntry("Icon"),
                                                 cg.readEntry("Exec"),
-                                                cg.readEntry("NoDisplay", false)));
+                                                cg.readEntry("NoDisplay", false), serviceClone));
             }
         } else {
             qCWarning(SERVICES) << "The desktop file" << q->entryPath() << "references the action" << group << "but doesn't define it";
@@ -392,6 +398,11 @@ KService::KService(const KDesktopFile *config, const QString &entryPath)
 KService::KService(QDataStream &_str, int _offset)
     : KSycocaEntry(*new KServicePrivate(_str, _offset))
 {
+    Q_D(KService);
+    KService::Ptr serviceClone(new KService(*this));
+    for (KServiceAction &action : d->m_actions) {
+        action.setService(serviceClone);
+    }
 }
 
 KService::KService(const KService &other)
@@ -470,7 +481,7 @@ bool KService::hasMimeType(const QString &mimeType) const
         if (st == mime) {
             return true;
         }
-        // TODO: should we handle inherited mimetypes here?
+        // TODO: should we handle inherited MIME types here?
         // KMimeType was in kio when this code was written, this is the only reason it's not done.
         // But this should matter only in a very rare case, since most code gets KServices from ksycoca.
         // Warning, change hasServiceType if you implement this here (and check kbuildservicefactory).
@@ -873,8 +884,8 @@ QString KService::exec() const
 {
     Q_D(const KService);
     if (d->m_strType == QLatin1String("Application") && d->m_strExec.isEmpty()) {
-        qCWarning(SERVICES) << "The desktop entry file " << entryPath()
-                   << " has Type=" << d->m_strType << " but has no Exec field.";
+        qCWarning(SERVICES) << "The desktop entry file" << entryPath()
+                   << "has Type=" << d->m_strType << "but has no Exec field.";
     }
     return d->m_strExec;
 }
