@@ -1,21 +1,9 @@
-/* This file is part of the KDE project
-   Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
-   Copyright     1999-2006  David Faure <faure@kde.org>
+/*
+    This file is part of the KDE project
+    SPDX-FileCopyrightText: 1998, 1999 Torben Weis <weis@kde.org>
+    SPDX-FileCopyrightText: 1999-2006 David Faure <faure@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #ifndef KSERVICEPRIVATE_H
@@ -40,6 +28,7 @@ public:
     {
         load(_str);
     }
+    KServicePrivate(const KServicePrivate& other) = default;
 
     void init(const KDesktopFile *config, KService *q);
 
@@ -80,7 +69,7 @@ public:
     QString m_strExec;
     QString m_strIcon;
     QString m_strTerminalOptions;
-    QString m_strPath;
+    QString m_strWorkingDirectory;
     QString m_strComment;
     QString m_strLibrary;
 
