@@ -1,21 +1,9 @@
-/* This file is part of the KDE libraries
-   Copyright (C) 2000 Torben Weis <weis@kde.org>
-   Copyright (C) 2006 David Faure <faure@kde.org>
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2000 Torben Weis <weis@kde.org>
+    SPDX-FileCopyrightText: 2006 David Faure <faure@kde.org>
 
-   This library is free software; you can redistribute it and/or
-   modify it under the terms of the GNU Library General Public
-   License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
-
-   This library is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Library General Public License for more details.
-
-   You should have received a copy of the GNU Library General Public License
-   along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kservicetypetrader.h"
@@ -88,16 +76,6 @@ void KServiceTypeTrader::applyConstraints(KService::List &lst,
     }
 }
 
-#if 0
-static void dumpOfferList(const KServiceOfferList &offers)
-{
-    // qDebug() << "Sorted list:";
-    OfferList::Iterator itOff = offers.begin();
-    for (; itOff != offers.end(); ++itOff)
-        // qDebug() << (*itOff).service()->name() << " allow-as-default=" << (*itOff).allowAsDefault() << " preference=" << (*itOff).preference();
-    }
-#endif
-
 KServiceOfferList KServiceTypeTrader::weightedOffers(const QString &serviceType)   // static, internal
 {
     //qDebug() << "KServiceTypeTrader::weightedOffers( " << serviceType << " )";
@@ -116,11 +94,6 @@ KServiceOfferList KServiceTypeTrader::weightedOffers(const QString &serviceType)
     const KServiceOfferList services = KSycocaPrivate::self()->serviceFactory()->offers(servTypePtr->offset(), servTypePtr->serviceOffersOffset());
 
     const KServiceOfferList offers = KServiceTypeProfile::sortServiceTypeOffers(services, serviceType);
-    //qDebug() << "Found profile: " << offers.count() << " offers";
-
-#if 0
-    dumpOfferList(offers);
-#endif
 
     return offers;
 }
@@ -179,10 +152,14 @@ KService::Ptr KServiceTypeTrader::preferredService(const QString &serviceType) c
     const KServiceOfferList offers = weightedOffers(serviceType);
 
     KServiceOfferList::const_iterator itOff = offers.begin();
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 67)
     // Look for the first one that is allowed as default.
     // Since the allowed-as-default are first anyway, we only have
     // to look at the first one to know.
     if (itOff != offers.end() && (*itOff).allowAsDefault()) {
+#else
+    if (itOff != offers.end()) {
+#endif
         return (*itOff).service();
     }
 
