@@ -1,20 +1,9 @@
-/*  This file is part of the KDE libraries
- *  Copyright (C) 2000 Waldo Bastian <bastian@kde.org>
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License version 2 as published by the Free Software Foundation.
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
- **/
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 2000 Waldo Bastian <bastian@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-only
+*/
 
 #include "kbuildservicegroupfactory_p.h"
 #include "ksycoca.h"
@@ -26,19 +15,16 @@
 #include <QDebug>
 #include <assert.h>
 #include <QHash>
+#include <QIODevice>
 
 KBuildServiceGroupFactory::KBuildServiceGroupFactory(KSycoca *db)
     : KServiceGroupFactory(db)
 {
-    m_resourceList = new KSycocaResourceList;
-//   m_resourceList->add( "apps", "*.directory" );
-
     m_baseGroupDict = new KSycocaDict();
 }
 
 KBuildServiceGroupFactory::~KBuildServiceGroupFactory()
 {
-    delete m_resourceList;
 }
 
 KServiceGroup *KBuildServiceGroupFactory::createEntry(const QString &) const
@@ -110,7 +96,7 @@ KBuildServiceGroupFactory::addNew(const QString &menuName, const QString &file,
 void
 KBuildServiceGroupFactory::addNewChild(const QString &parent, const KSycocaEntry::Ptr &newEntry)
 {
-    QString name = QStringLiteral("#parent#") + parent;
+    QString name = QLatin1String("#parent#") + parent;
 
     KServiceGroup::Ptr entry;
     KSycocaEntry::Ptr ptr = m_entryDict->value(name);
