@@ -1,44 +1,33 @@
-/*  This file is part of the KDE libraries
- *  Copyright (C) 1999-2000 Waldo Bastian <bastian@kde.org>
- *  Copyright (C) 2005-2009 David Faure <faure@kde.org>
- *  Copyright (C) 2008 Hamish Rodda <rodda@kde.org>
- *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License version 2 as published by the Free Software Foundation;
- *
- *  This library is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *  Library General Public License for more details.
- *
- *  You should have received a copy of the GNU Library General Public License
- *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
- **/
+/*
+    This file is part of the KDE libraries
+    SPDX-FileCopyrightText: 1999-2000 Waldo Bastian <bastian@kde.org>
+    SPDX-FileCopyrightText: 2005-2009 David Faure <faure@kde.org>
+    SPDX-FileCopyrightText: 2008 Hamish Rodda <rodda@kde.org>
+    SPDX-FileCopyrightText: 2020 Harald Sitter <sitter@kde.org>
+
+    SPDX-License-Identifier: LGPL-2.0-only
+*/
 
 #include "ksycoca.h"
 #include "ksycoca_p.h"
 #include "ksycocautils_p.h"
 #include "ksycocatype.h"
 #include "ksycocafactory_p.h"
-#include "kconfiggroup.h"
-#include "ksharedconfig.h"
+#include <KConfigGroup>
+#include <KSharedConfig>
 #include "sycocadebug.h"
 
-#include <qstandardpaths.h>
-#include <QtCore/QDataStream>
-#include <QtCore/QCoreApplication>
-#include <QtCore/QFile>
-#include <QtCore/QFileInfo>
+#include <QStandardPaths>
+#include <QDataStream>
+#include <QCoreApplication>
+#include <QFile>
+#include <QFileInfo>
 #include <QThread>
 #include <QThreadStorage>
 #include <QMetaMethod>
 
 #include <stdlib.h>
 #include <fcntl.h>
-#include <QDir>
 #include <kmimetypefactory_p.h>
 #include <kservicetypefactory_p.h>
 #include <kservicegroupfactory_p.h>
@@ -60,7 +49,7 @@
  * However running apps should still be able to read it, so
  * only add to the data, never remove/modify.
  */
-#define KSYCOCA_VERSION 303
+#define KSYCOCA_VERSION 304
 
 #if HAVE_MADVISE || HAVE_MMAP
 #include <sys/mman.h> // This #include was checked when looking for posix_madvise
@@ -91,17 +80,17 @@ KSycocaPrivate::KSycocaPrivate(KSycoca *q)
       timeStamp(0),
       m_databasePath(),
       updateSig(0),
+      m_fileWatcher(new KDirWatch),
       m_haveListeners(false),
-      m_globalDatabase(false),
       q(q),
       sycoca_size(0),
-      sycoca_mmap(0),
-      m_mmapFile(0),
-      m_device(0),
-      m_mimeTypeFactory(0),
-      m_serviceTypeFactory(0),
-      m_serviceFactory(0),
-      m_serviceGroupFactory(0)
+      sycoca_mmap(nullptr),
+      m_mmapFile(nullptr),
+      m_device(nullptr),
+      m_mimeTypeFactory(nullptr),
+      m_serviceTypeFactory(nullptr),
+      m_serviceFactory(nullptr),
+      m_serviceGroupFactory(nullptr)
 {
 #ifdef Q_OS_WIN
     /*
@@ -141,14 +130,14 @@ bool KSycocaPrivate::tryMmap()
     }
     fcntl(m_mmapFile->handle(), F_SETFD, FD_CLOEXEC);
     sycoca_size = m_mmapFile->size();
-    void *mmapRet = mmap(0, sycoca_size,
+    void *mmapRet = mmap(nullptr, sycoca_size,
                        PROT_READ, MAP_SHARED,
                        m_mmapFile->handle(), 0);
     /* POSIX mandates only MAP_FAILED, but we are paranoid so check for
        null pointer too.  */
-    if (mmapRet == MAP_FAILED || mmapRet == 0) {
+    if (mmapRet == MAP_FAILED || mmapRet == nullptr) {
         qCDebug(SYCOCA).nospace() << "mmap failed. (length = " << sycoca_size << ")";
-        sycoca_mmap = 0;
+        sycoca_mmap = nullptr;
         return false;
     } else {
         sycoca_mmap = static_cast<const char *>(mmapRet);
@@ -199,26 +188,18 @@ QString KSycocaPrivate::findDatabase()
 {
     Q_ASSERT(databaseStatus == DatabaseNotOpen);
 
-    m_globalDatabase = false;
-    QString path = KSycoca::absoluteFilePath();
-    QFileInfo info(path);
-    bool canRead = info.isReadable();
-    if (!canRead) {
-        path = KSycoca::absoluteFilePath(KSycoca::GlobalDatabase);
-        if (!path.isEmpty()) {
-            info.setFile(path);
-            canRead = info.isReadable();
-            if (canRead) {
-                m_globalDatabase = true;
-            }
-        }
-    }
-    if (canRead) {
-        if (m_haveListeners) {
-            m_fileWatcher.addFile(path);
+    const QString path = KSycoca::absoluteFilePath();
+    const QFileInfo info(path);
+    if (info.isReadable()) {
+        if (m_haveListeners && m_fileWatcher) {
+            m_fileWatcher->addFile(path);
         }
         return path;
     }
+    // Let's be notified when it gets created - by another process or by ourselves
+    if (m_fileWatcher) {
+        m_fileWatcher->addFile(path);
+    }
     return QString();
 }
 
@@ -227,14 +208,19 @@ QString KSycocaPrivate::findDatabase()
 KSycoca::KSycoca()
     : d(new KSycocaPrivate(this))
 {
-    connect(&d->m_fileWatcher, &KDirWatch::created, this, [this](){ d->slotDatabaseChanged(); });
+    if (d->m_fileWatcher) {
+        // We always delete and recreate the DB, so KDirWatch normally emits created
+        connect(d->m_fileWatcher.get(), &KDirWatch::created, this, [this](){ d->slotDatabaseChanged(); });
+        // In some cases, KDirWatch only thinks the file was modified though
+        connect(d->m_fileWatcher.get(), &KDirWatch::dirty, this, [this]() { d->slotDatabaseChanged(); });
+    }
 }
 
-bool KSycocaPrivate::openDatabase(bool openDummyIfNotFound)
+bool KSycocaPrivate::openDatabase()
 {
     Q_ASSERT(databaseStatus == DatabaseNotOpen);
 
-    delete m_device; m_device = 0;
+    delete m_device; m_device = nullptr;
 
     if (m_databasePath.isEmpty()) {
         m_databasePath = findDatabase();
@@ -244,21 +230,10 @@ bool KSycocaPrivate::openDatabase(bool openDummyIfNotFound)
     if (!m_databasePath.isEmpty()) {
         qCDebug(SYCOCA) << "Opening ksycoca from" << m_databasePath;
         m_dbLastModified = QFileInfo(m_databasePath).lastModified();
-        checkVersion();
+        result = checkVersion();
     } else { // No database file
         //qCDebug(SYCOCA) << "Could not open ksycoca";
-        m_databasePath.clear();
-        if (openDummyIfNotFound) {
-            // We open a dummy database instead.
-            //qCDebug(SYCOCA) << "No database, opening a dummy one.";
-
-            m_sycocaStrategy = StrategyDummyBuffer;
-            QDataStream *str = stream();
-            *str << qint32(KSYCOCA_VERSION);
-            *str << qint32(0);
-        } else {
-            result = false;
-        }
+        result = false;
     }
     return result;
 }
@@ -269,36 +244,30 @@ KSycocaAbstractDevice *KSycocaPrivate::device()
         return m_device;
     }
 
-    Q_ASSERT(!m_databasePath.isEmpty());
-
     KSycocaAbstractDevice *device = m_device;
-    if (m_sycocaStrategy == StrategyDummyBuffer) {
-        device = new KSycocaBufferDevice;
-        device->device()->open(QIODevice::ReadOnly); // can't fail
-    } else {
+    Q_ASSERT(!m_databasePath.isEmpty());
 #if HAVE_MMAP
-        if (m_sycocaStrategy == StrategyMmap && tryMmap()) {
-            device = new KSycocaMmapDevice(sycoca_mmap,
-                                           sycoca_size);
-            if (!device->device()->open(QIODevice::ReadOnly)) {
-                delete device; device = 0;
-            }
+    if (m_sycocaStrategy == StrategyMmap && tryMmap()) {
+        device = new KSycocaMmapDevice(sycoca_mmap,
+                sycoca_size);
+        if (!device->device()->open(QIODevice::ReadOnly)) {
+            delete device; device = nullptr;
         }
+    }
 #endif
 #ifndef QT_NO_SHAREDMEMORY
-        if (!device && m_sycocaStrategy == StrategyMemFile) {
-            device = new KSycocaMemFileDevice(m_databasePath);
-            if (!device->device()->open(QIODevice::ReadOnly)) {
-                delete device; device = 0;
-            }
+    if (!device && m_sycocaStrategy == StrategyMemFile) {
+        device = new KSycocaMemFileDevice(m_databasePath);
+        if (!device->device()->open(QIODevice::ReadOnly)) {
+            delete device; device = nullptr;
         }
+    }
 #endif
-        if (!device) {
-            device = new KSycocaFileDevice(m_databasePath);
-            if (!device->device()->open(QIODevice::ReadOnly)) {
-                qCWarning(SYCOCA) << "Couldn't open" << m_databasePath << "even though it is readable? Impossible.";
-                //delete device; device = 0; // this would crash in the return statement...
-            }
+    if (!device) {
+        device = new KSycocaFileDevice(m_databasePath);
+        if (!device->device()->open(QIODevice::ReadOnly)) {
+            qCWarning(SYCOCA) << "Couldn't open" << m_databasePath << "even though it is readable? Impossible.";
+            //delete device; device = 0; // this would crash in the return statement...
         }
     }
     if (device) {
@@ -311,7 +280,7 @@ QDataStream *&KSycocaPrivate::stream()
 {
     if (!m_device) {
         if (databaseStatus == DatabaseNotOpen) {
-            checkDatabase(KSycocaPrivate::IfNotFoundRecreate | KSycocaPrivate::IfNotFoundOpenDummy);
+            checkDatabase(KSycocaPrivate::IfNotFoundRecreate);
         }
 
         device(); // create m_device
@@ -326,9 +295,9 @@ void KSycocaPrivate::slotDatabaseChanged()
     changeList = QStringList() << QStringLiteral("services") << QStringLiteral("servicetypes") << QStringLiteral("xdgdata-mime") << QStringLiteral("apps");
 
     qCDebug(SYCOCA) << QThread::currentThread() << "got a notifyDatabaseChanged signal";
-    // kbuildsycoca tells us the database file changed
+    // KDirWatch tells us the database file changed
     // We would have found out in the next call to ensureCacheValid(), but for
-    // now keep the call to closeDatabase, to help refcounting to 0 the old mmaped file earlier.
+    // now keep the call to closeDatabase, to help refcounting to 0 the old mmapped file earlier.
     closeDatabase();
     // Start monitoring the new file right away
     m_databasePath = findDatabase();
@@ -401,13 +370,13 @@ KSycoca::~KSycoca()
 
 bool KSycoca::isAvailable() // TODO KF6: make it non-static (mostly useful for unittests)
 {
-    return self()->d->checkDatabase(KSycocaPrivate::IfNotFoundDoNothing/* don't open dummy db if not found */);
+    return self()->d->checkDatabase(KSycocaPrivate::IfNotFoundDoNothing);
 }
 
 void KSycocaPrivate::closeDatabase()
 {
     delete m_device;
-    m_device = 0;
+    m_device = nullptr;
 
     // It is very important to delete all factories here
     // since they cache information about the database file
@@ -416,19 +385,19 @@ void KSycocaPrivate::closeDatabase()
     qDeleteAll(m_factories);
     m_factories.clear();
 
-    m_mimeTypeFactory = 0;
-    m_serviceFactory = 0;
-    m_serviceTypeFactory = 0;
-    m_serviceGroupFactory = 0;
+    m_mimeTypeFactory = nullptr;
+    m_serviceFactory = nullptr;
+    m_serviceTypeFactory = nullptr;
+    m_serviceGroupFactory = nullptr;
 
 #if HAVE_MMAP
     if (sycoca_mmap) {
         // Solaris has munmap(char*, size_t) and everything else should
         // be happy with a char* for munmap(void*, size_t)
         munmap(const_cast<char *>(sycoca_mmap), sycoca_size);
-        sycoca_mmap = 0;
+        sycoca_mmap = nullptr;
     }
-    delete m_mmapFile; m_mmapFile = 0;
+    delete m_mmapFile; m_mmapFile = nullptr;
 #endif
 
     databaseStatus = DatabaseNotOpen;
@@ -441,7 +410,7 @@ void KSycoca::addFactory(KSycocaFactory *factory)
     d->addFactory(factory);
 }
 
-#ifndef KSERVICE_NO_DEPRECATED
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 0)
 bool KSycoca::isChanged(const char *type)
 {
     return self()->d->changeList.contains(QString::fromLatin1(type));
@@ -501,21 +470,19 @@ bool KSycocaPrivate::checkDatabase(BehaviorsIfNotFound ifNotFound)
     closeDatabase(); // close the dummy one
 
     // Check if new database already available
-    if (openDatabase(ifNotFound & IfNotFoundOpenDummy)) {
-        if (checkVersion()) {
-            // Database exists, and version is ok, we can read it.
+    if (openDatabase()) {
+        // Database exists, and version is ok, we can read it.
 
-            if (qAppName() != KBUILDSYCOCA_EXENAME && ifNotFound != IfNotFoundDoNothing) {
+        if (qAppName() != QLatin1String(KBUILDSYCOCA_EXENAME) && ifNotFound != IfNotFoundDoNothing) {
 
-                // Ensure it's uptodate, rebuild if needed
-                checkDirectories();
+            // Ensure it's uptodate, rebuild if needed
+            checkDirectories();
 
-                // Don't check again for some time
-                m_lastCheck.start();
-            }
-
-            return true;
+            // Don't check again for some time
+            m_lastCheck.start();
         }
+
+        return true;
     }
 
     if (ifNotFound & IfNotFoundRecreate) {
@@ -529,7 +496,7 @@ QDataStream *KSycoca::findFactory(KSycocaFactoryId id)
 {
     // Ensure we have a valid database (right version, and rewinded to beginning)
     if (!d->checkDatabase(KSycocaPrivate::IfNotFoundRecreate)) {
-        return 0;
+        return nullptr;
     }
 
     QDataStream *str = stream();
@@ -550,7 +517,7 @@ QDataStream *KSycoca::findFactory(KSycocaFactoryId id)
             return str;
         }
     }
-    return 0;
+    return nullptr;
 }
 
 bool KSycoca::needsRebuild()
@@ -590,6 +557,15 @@ KSycocaHeader KSycocaPrivate::readSycocaHeader()
         allResourceDirs.insert(directoryList.at(i), mtime);
     }
 
+    QStringList fileList;
+    *str >> fileList;
+    extraFiles.clear();
+    for (auto fileName : fileList) {
+        qint64 mtime;
+        *str >> mtime;
+        extraFiles.insert(fileName, mtime);
+    }
+
     str->device()->seek(oldPos);
 
     timeStamp = header.timeStamp;
@@ -598,16 +574,6 @@ KSycocaHeader KSycocaPrivate::readSycocaHeader()
     language = header.language;
     updateSig = header.updateSignature;
 
-    if (m_globalDatabase) {
-        // The global database doesn't point to the user's local dirs, but we need to check them too
-        // to react on something being created there
-        const QString dataHome = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation);
-        addLocalResourceDir(dataHome + QLatin1String("/kservices5"));
-        addLocalResourceDir(dataHome + QLatin1String("/kservicetypes5"));
-        addLocalResourceDir(dataHome + QLatin1String("/mime"));
-        addLocalResourceDir(dataHome + QLatin1String("/applications"));
-    }
-
     return header;
 }
 
@@ -622,10 +588,10 @@ public:
     // Check times of last modification of all directories on which ksycoca depends,
     // If none of them is newer than the mtime we stored for that directory at the
     // last rebuild, this means that there's no need to rebuild ksycoca.
-    bool checkTimestamps(const QMap<QString, qint64> &dirs)
+    bool checkDirectoriesTimestamps(const QMap<QString, qint64> &dirs)
     {
         Q_ASSERT(!dirs.isEmpty());
-        qCDebug(SYCOCA) << "checking file timestamps";
+        //qCDebug(SYCOCA) << "checking file timestamps";
         for (auto it = dirs.begin(); it != dirs.end(); ++it) {
             const QString dir = it.key();
             const qint64 lastStamp = it.value();
@@ -636,7 +602,7 @@ public:
                     if (mtime > m_now) {
                         qCDebug(SYCOCA) << fi.filePath() << "has a modification time in the future" << mtime;
                     }
-                    qCDebug(SYCOCA) << "timestamp changed:" << fi.filePath() << mtime << ">" << QDateTime::fromMSecsSinceEpoch(lastStamp);
+                    qCDebug(SYCOCA) << "dir timestamp changed:" << fi.filePath() << mtime << ">" << QDateTime::fromMSecsSinceEpoch(lastStamp);
                     // no need to continue search
                     return false;
                 }
@@ -651,6 +617,28 @@ public:
         return true;
     }
 
+    bool checkFilesTimestamps(const QMap<QString, qint64> &files)
+    {
+        for (auto it = files.begin(); it != files.end(); ++it) {
+            const QString fileName = it.key();
+            const qint64 lastStamp = it.value();
+
+            QFileInfo fi(fileName);
+            if (!fi.exists()) {
+                return false;
+            }
+            const QDateTime mtime = fi.lastModified();
+            if (mtime.toMSecsSinceEpoch() > lastStamp) {
+                if (mtime > m_now) {
+                    qCDebug(SYCOCA) << fi.filePath() << "has a modification time in the future" << mtime;
+                }
+                qCDebug(SYCOCA) << "file timestamp changed:" << fi.filePath() << mtime << ">" << QDateTime::fromMSecsSinceEpoch(lastStamp);
+                return false;
+            }
+        }
+        return true;
+    }
+
 private:
     QDateTime m_now;
 };
@@ -669,7 +657,18 @@ bool KSycocaPrivate::needsRebuild()
     }
     // these days timeStamp is really a "bool headerFound", the value itself doesn't matter...
     // KF6: replace it with bool.
-    return timeStamp != 0 && !TimestampChecker().checkTimestamps(allResourceDirs);
+    bool ret = timeStamp != 0 &&
+            (!TimestampChecker().checkDirectoriesTimestamps(allResourceDirs)
+            ||
+            !TimestampChecker().checkFilesTimestamps(extraFiles));
+    if (ret) {
+        return true;
+    }
+    auto files = KBuildSycoca::factoryExtraFiles();
+    // ensure files are ordered so next comparison works
+    files.sort();
+    // to cover cases when extra files were added
+    return extraFiles.keys() != files;
 }
 
 bool KSycocaPrivate::buildSycoca()
@@ -684,15 +683,12 @@ bool KSycocaPrivate::buildSycoca()
     // Ok, the new database should be here now, open it.
     if (!openDatabase()) {
         qCDebug(SYCOCA) << "Still no database...";
-        return false; // Still no database - uh oh
-    }
-    if (!checkVersion()) {
-        qCDebug(SYCOCA) << "Still outdated...";
-        return false; // Still outdated - uh oh
+        return false;
     }
     return true;
 }
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 15)
 quint32 KSycoca::timeStamp()
 {
     if (!d->timeStamp) {
@@ -700,7 +696,9 @@ quint32 KSycoca::timeStamp()
     }
     return d->timeStamp / 1000; // from ms to s
 }
+#endif
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 15)
 quint32 KSycoca::updateSignature()
 {
     if (!d->timeStamp) {
@@ -708,41 +706,30 @@ quint32 KSycoca::updateSignature()
     }
     return d->updateSig;
 }
+#endif
 
 QString KSycoca::absoluteFilePath(DatabaseType type)
 {
+    Q_UNUSED(type); // GlobalDatabase concept removed in 5.61
     const QStringList paths = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
     QString suffix = QLatin1Char('_') + QLocale().bcp47Name();
 
-    if (type == GlobalDatabase) {
-        const QString fileName = QStringLiteral("ksycoca5") + suffix;
-        QString path = QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("ksycoca/") + fileName);
-        if (path.isEmpty()) {
-            // No existing global DB found, maybe we are going to make a new one.
-            // We don't want it in $HOME, so skip the first entry in <paths> and pick the second one.
-            if (paths.count() == 1) { // unlikely
-                return QString();
-            }
-            return paths.at(1) + QStringLiteral("/ksycoca/") + fileName;
-        }
-        return path;
-    }
-
     const QByteArray ksycoca_env = qgetenv("KDESYCOCA");
     if (ksycoca_env.isEmpty()) {
-        const QByteArray pathHash = QCryptographicHash::hash(paths.join(QString(QLatin1Char(':'))).toUtf8(), QCryptographicHash::Sha1);
+        const QByteArray pathHash = QCryptographicHash::hash(paths.join(QLatin1Char(':')).toUtf8(), QCryptographicHash::Sha1);
         suffix += QLatin1Char('_') + QString::fromLatin1(pathHash.toBase64());
-        suffix.replace('/', '_');
+        suffix.replace(QLatin1Char('/'), QLatin1Char('_'));
 #ifdef Q_OS_WIN
-        suffix.replace(':', '_');
+        suffix.replace(QLatin1Char(':'), QLatin1Char('_'));
 #endif
-        const QString fileName = QStringLiteral("ksycoca5") + suffix;
+        const QString fileName = QLatin1String("ksycoca5") + suffix;
         return QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation) + QLatin1Char('/') + fileName;
     } else {
         return QFile::decodeName(ksycoca_env);
     }
 }
 
+#if KSERVICE_BUILD_DEPRECATED_SINCE(5, 15)
 QString KSycoca::language()
 {
     if (d->language.isEmpty()) {
@@ -750,6 +737,7 @@ QString KSycoca::language()
     }
     return d->language;
 }
+#endif
 
 QStringList KSycoca::allResourceDirs()
 {
@@ -767,7 +755,7 @@ void KSycoca::flagError()
         return;
     }
     sycoca->d->readError = true;
-    if (qAppName() != KBUILDSYCOCA_EXENAME && !sycoca->isBuilding()) {
+    if (qAppName() != QLatin1String(KBUILDSYCOCA_EXENAME) && !sycoca->isBuilding()) {
         // Rebuild the damned thing.
         KBuildSycoca builder;
         (void)builder.recreate();
@@ -781,7 +769,7 @@ bool KSycoca::isBuilding()
 
 void KSycoca::disableAutoRebuild()
 {
-    qCWarning(SYCOCA) << "KSycoca::disableAutoRebuild() is internal, do not call it.";
+    ksycocaInstance->sycoca()->d->m_fileWatcher = nullptr;
 }
 
 QDataStream *&KSycoca::stream()
@@ -795,8 +783,8 @@ void KSycoca::connectNotify(const QMetaMethod &signal)
         d->m_haveListeners = true;
         if (d->m_databasePath.isEmpty()) {
             d->m_databasePath = d->findDatabase();
-        } else {
-            d->m_fileWatcher.addFile(d->m_databasePath);
+        } else if (d->m_fileWatcher) {
+            d->m_fileWatcher->addFile(d->m_databasePath);
         }
     }
 }
@@ -812,7 +800,7 @@ KSERVICE_EXPORT int ksycoca_ms_between_checks = 1500;
 
 void KSycoca::ensureCacheValid()
 {
-    if (qAppName() == KBUILDSYCOCA_EXENAME) {
+    if (qAppName() == QLatin1String(KBUILDSYCOCA_EXENAME)) {
         return;
     }
 
